#include <math.h>
#include <string.h>
#include <uf.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_obj.h>
#include <uf_drf.h>
#include <uf_assem.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_dimension_button_edges 
*
* Obtain the three edges for the button dimension.  They are the left
* front, right front and right rear vertical edges.
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                   
*   01   14FEB03   DKF   Removed unreferenced local variables,
*                        Updated error checking
*
* input 
*  (none)  
*
* output 
* edge                            array of edges 
* epcode                          array of edge codes 
*               UF_DRF_first_end_point = dimension to start point 
*                UF_DRF_last_end_point = dimension to end point 
*  calc_dimension_button_edges       0 = success, >0 = error 
*  
**********************************************************************/

    int calc_dimension_button_edges(tag_t *edge, int *epcode)
{
/******************** Routine variable declaration  *******************/
    int ecount,         /* count of edges */
        vcount,         /* count of vertices */
        i;              /* loop variable */

    tag_t
         body,          /* solid body */
         edge_tag;      /* any edge */

    uf_list_p_t
        elist;          /* list of edges */

    double
        pt1[3],         /* start point of edge */
        pt2[3],         /* end point of edge */
        border,         /* height or width of button border */
        button_height,  /* button height */
        button_width;   /* button width */

/******************** Beginning of Executable code *********************/


/* Evaluate the expressions for the button width/height and border.
 * Check the return codes.
 */
    UF_CALL(UF_MODL_eval_exp("Border",&border));
    UF_CALL(UF_MODL_eval_exp("Button_height",&button_height));
    UF_CALL(UF_MODL_eval_exp("Button_width",&button_width));

/* Locate the target body using it's name;  "BUTTONONE"  */
    body = NULL_TAG;
    UF_CALL( UF_OBJ_cycle_by_name("BUTTONONE",&body) );

/* Obtain the edges and count of edges in the body.  */
    UF_CALL(UF_MODL_ask_body_edges(body,&elist));
    UF_CALL(UF_MODL_ask_list_count(elist,&ecount));

/* Loop through the edges and find the vertices.  Check for specific
 * edges and store those particular edges and the appropriate end
 * point code.  A code of UF_DRF_first_end_point is if the first vertex
 * is the desired endpoint;  code UF_DRF_last_end_point is for the
 * second endpoint.
 */

    for(i = 0; i < ecount; i++) {
        UF_CALL(UF_MODL_ask_list_item(elist,i,&edge_tag));
        UF_CALL(UF_MODL_ask_edge_verts(edge_tag,pt1,pt2,&vcount));

/* Look for the Z edges */
        if(fabs(fabs(pt1[2] - pt2[2]) - 5.0) < 0.001) {

/* look for left front Z edge */
            if((fabs(pt1[0] - border) < 0.001)
              && (fabs(pt1[1] - border) < 0.001)) {
                edge[0] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[0] = UF_DRF_first_end_point;
                else
                    epcode[0] = UF_DRF_last_end_point;
                continue;
            }

/* look for right front Z edge */
            if((fabs(pt1[0] - (border+button_width)) < 0.001)
              && (fabs(pt1[1] - border) < 0.001)) {
                edge[1] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[1] = UF_DRF_first_end_point;
                else
                    epcode[1] = UF_DRF_last_end_point;
                continue;
            }

/* look for right rear edge */
            if((fabs(pt1[0] - (border+button_width)) < 0.001)
              && (fabs(pt1[1] - (border+button_height)) < 0.001)) {
                edge[2] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[2] = UF_DRF_first_end_point;
                else
                    epcode[2] = UF_DRF_last_end_point;
                continue;
            }
        }           /* endif */
    }               /* endloop */

/* Delete the list of edges before returning.  */
    UF_CALL(UF_MODL_delete_list(&elist));

    return (0);
}
