#include <stdlib.h>
#include <uf_defs.h>
#include <uf_layout.h>
#include <uf_modl.h>
#include <uf_csys.h>
#include <uf_view.h>
#include <uf_obj.h>
#include <uf_drf.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_dimension_button 
*
* Create dimensions on the button.  Use the routine that extracts edges
* to obtain the objects for dimensioning.  Edges are returned in a 
* specific order.
*     edge[0]   Button left front
*     edge[1]   Button right front
*     edge[2]   Button right rear
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                   
*   01   01JUL96   J_P   Replaced uf5205 call with UF_OBJ_delete_object
*   02   01SEP97   J_P   Removed delete_object calls.  The set_wcs
*                        routine uses temporary coordinate systems.
*   03   14FEB03   DKF   Added uf6460 call to create 4 view drf layout,
*                        Updated error checking
*
* input 
*  (none)  
*
* output 
*  calc_dimension_button       0 = success, >0 = error 
*  
*  
**********************************************************************/
    int calc_dimension_button()
{
/******************** Routine variable declaration  *******************/
    char
        radsym[7],      /* radius symbol string */
        diasym[7];      /* diameter symbol string */

    int mpi[100],       /* integer drafting parameters */
        epcode[3],      /* startpoint-or-endpoint flags*/
		scale_option=2, /* fit all views in uc6460, uc6468 */
		layout_style=4,	/* 4 view layout for uc6460 */
        flag;           /* results from functions */

    tag_t
        edge[3],            /* edges to be dimensioned */
        view_tag,           /* Tag of the view containing dimensions */
        dimension,          /* dimension object */
        csys1;              /* Original work coordinate system. */

    double
        mpr[70],        /* floating point drafting parameters */
        origin[3],      /* dimension origin */
        border,         /* border distance */
        button_height,  /* height of calculator */
        button_width,   /* width of calculator */
		scale=1;		/* for uc6460, uc6468 */

    UF_DRF_text_t
        dim_text;       /* Struct. for appended text and man. dim */

    UF_DRF_object_t
        dim0,           /* Structure for first object in dim. */
        dim1;           /* Structure for second object in dim. */

	const char
		layout_name[31]="L4_DRF",
		layout_views[][31]={"TOP_DRF","FRONT_DRF","RIGHT_DRF","TFR-TRI"};

/******************** Beginning of Executable code *********************/


/* Read the current settings for drafting/dimensioning */
    UF_CALL( UF_DRF_ask_preferences( mpi, mpr, radsym, diasym ) );

/* Set the linear units to mm and change the character size and number
 * of decimal places.  Call the routine that reset draft/dim settings.
 */
    mpi[3]  = 2;        /* decimal places */
    mpi[13] = 1;        /* linear units */
    mpr[32] = 5.0;      /* character height */
    UF_CALL( UF_DRF_set_preferences( mpi, mpr, radsym, diasym ) );

/* Evaluate the expressions for the button height and width and
 * the border distance.  Check return codes.
 */
    UF_CALL( UF_MODL_eval_exp("Button_height", &button_height) );
    UF_CALL( UF_MODL_eval_exp("Button_width", &button_width) );
    UF_CALL( UF_MODL_eval_exp("Border", &border) );
 
/* Initialize structures.  This sets the associativity modifiers
 * to UF_DRF_end_point (which is what we want.  Check return codes!
 * Also initialize the text structure.
 */
    UF_CALL( UF_DRF_init_object_structure( &dim0 ) );
    UF_CALL( UF_DRF_init_object_structure( &dim1 ) );
    dim_text.lines_app_text = 0;
    dim_text.user_dim_text = NULL;
    dim_text.appended_text = NULL;

/* Get the edges for the dimensions.  They are returned in a particular
 * order documented in the comments at the routine start.
 */
    flag = calc_dimension_button_edges(edge,epcode);
    if(flag) return (flag);

/* Section to create horizontal dimensions */

/* Set the work view (and WCS) to create a dimension in the FRONT
 * view.  Save the original WCS in csys1.
 */
    UF_CALL( uc6464("","","FRONT") );
    UF_CALL( UF_CSYS_ask_wcs(&csys1) );
    flag = ug_set_wcs_to_view("FRONT");
    if(flag) return(flag);

/* save the front view to a custom view "front_drf" */
/* then do the dimensioning in this new view    */

   UF_CALL( UF_VIEW_ask_work_view(&view_tag) );
   UF_CALL( UF_VIEW_copy_view(view_tag,"FRONT_DRF",&view_tag) );
   UF_CALL( uc6464("","","FRONT_DRF") );

/* Obtain the tag of the work view.  It is used in the structures. */

    UF_CALL( UF_VIEW_ask_tag_of_view_name( "", &view_tag ) );
    dim0.object_view_tag = dim1.object_view_tag = view_tag;

/* Set the object tags and assoc. modifiers.  Then create a dimension
 * from the left edge to right edge.
 */
    origin[0] = border + 0.5*button_width;
    origin[1] = 12.0 + 2.0*mpr[32];
    origin[2] = 0.0;
    dim0.object_assoc_modifier = epcode[0];
    dim0.object_tag = edge[0];
    dim1.object_assoc_modifier = epcode[1];
    dim1.object_tag = edge[1];
    UF_CALL( UF_DRF_create_horizontal_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );
    UF_CALL( UF_VIEW_fit_view(NULL_TAG,1.0) );
    UF_CALL( UF_VIEW_save_all_active_views() );

/* Set the work view (and WCS) to create a dimension in the RIGHT
 *  view.
 */
    UF_CALL(uc6464("","","RIGHT") );
    flag = ug_set_wcs_to_view("RIGHT");
    if(flag) return(flag);

/* save the right view to a custom view "RIGHT_DRF" */
/* then do the dimensioning in this new view    */

   UF_CALL( UF_VIEW_ask_work_view(&view_tag) );
   UF_CALL( UF_VIEW_copy_view(view_tag,"RIGHT_DRF",&view_tag) );
   UF_CALL( uc6464("","","RIGHT_DRF") );

/* Obtain the tag of the work view.  It is used in the structures. */
    UF_CALL( UF_VIEW_ask_tag_of_view_name( "", &view_tag ) );
    dim0.object_view_tag = dim1.object_view_tag = view_tag;

/* Set the object tags and assoc. modifiers.  Then create a dimension
 * from the front right edge to rear right edge.
 */
    origin[0] = border + 0.5*button_height;
    origin[1] = 12.0 + 2.0*mpr[32];
    origin[2] = 0.0;
    dim0.object_assoc_modifier = epcode[2];
    dim0.object_tag = edge[2];
    dim1.object_assoc_modifier = epcode[1];
    dim1.object_tag = edge[1];
    UF_CALL( UF_DRF_create_horizontal_dim( &dim0, &dim1,
             &dim_text, origin, &dimension ) );
    UF_CALL( UF_VIEW_fit_view(NULL_TAG,1.0) );
    UF_CALL( UF_VIEW_save_all_active_views() );

/* Create a drafting layout using the three dimensioned drafting views and TFR-TRI */
	UF_CALL( uc6460(layout_name, layout_style, layout_views, scale_option, scale) );

/* Retireve a single view layout */
	UF_CALL( uc6468("L1", scale_option, scale) );

/* Assure that the layout is set to the top view. */
    UF_CALL (uc6464("","","TOP") );


/* Reset the WCS to the original csys.  */
    UF_CALL( UF_CSYS_set_wcs( csys1) );

/* Fit the view.  Any temporary WCS's have been removed. */
    UF_CALL( UF_VIEW_ask_tag_of_view_name( "", &view_tag ) );
    UF_CALL( UF_VIEW_fit_view(view_tag, 1.) );

    return(0);
}
